# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 20th November 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis

##########################################
### Illustration: Convergence in probability
### and in distribution
##########################################
##########################################


##########################################
### Convergence in probability
### X_1, ... X_n iid Binomial(n, p)
### sum(X_i)/n in probability to p
##########################################
inprob.binom <- function(n, p = 1/2) {
  values <- 0:n
  means <- values/n
  probs <- dbinom(values, n, p)
  plot(means, probs, type = "h", col = "red",
       ylim = c(0, max(probs)),
       xlab = expression(bar(X)[n]),
       ylab = "Probability mass")
  points(x = c(p, p), y = c(0, 1), col = "blue", pch = 16)
  title(paste("n =",n,",","p = ",p))
}

inprob.binom(1, p = 1/4)

inprob.binom(10, p =1/4)

## nn takes values in the sequence 2^i, i = 2, ... , 15  
nns <- 2^(2:15)
for (nn in nns) {
	Sys.sleep(0.4)
	inprob.binom(nn, p = 1/4)
}	


##########################################
### Convergence in probability
### X_1, ... X_n iid Binomial(n, p)
### sum(X_i)/n in probability to p
###
### for the definition of convergence in
### probability give epsilon, delta
### and finds n_0.
###
### CAUTION!!!
### for very small epsilon, delta 
### inefficient (very naive implementation)
##########################################
find.n0 <- function(epsilon, delta, p = 1/2) {
  n <- 0
  test <- TRUE
  while (test) {
    n <- n + 1
    values <- 0:n
    means <- values/n
    Z <- means - p
    probs <- dbinom(values, n, p)
    prob.e <- sum(probs[which(Z <= epsilon)])
    prob.me <- sum(probs[which(Z < -epsilon)])
    if (1 - prob.e + prob.me > delta)
      test <- TRUE
    else
      test <- FALSE
  }
  list(n0 = n0)
}


##########################################
### Convergence in distribution
### X_1, ... X_n iid Binomial(n, p)
### sqrt(n)*(sum(X_i)/n - p)/sqrt(p*(1-p)) 
### in probability to N(0,1)
##########################################
indistr.binom <- function(n, p = 1/2) {
  values <- 0:n
  zvalues <- sqrt(n)*(values/n - p)/sqrt(p*(1-p))
  cumprobs <- c(0, pbinom(values, n, p))
  probs <- dbinom(values, n, p)
  plot(zz <- seq(-6, 6, 0.01), pnorm(zz), 
       type = "l", lwd = 3, col = "grey",
       xlab = expression(z == sqrt(n)(bar(X)[n] - p)/sqrt(p(1-p))),
       ylab = expression(F(z)))
  title(paste("n =",n,",","p = ",p))
  plot(stepfun(zvalues, cumprobs),add = TRUE,
       xlim = c(-6,6), do.points = FALSE)
}


indistr.binom(1, p = 1/4)

indistr.binom(5, p = 1/4)

## nn takes values in the sequence 2^i, i = 2, ... , 15  
nns <- 2^(2:15)
for (nn in nns) {
	Sys.sleep(0.2)
	indistr.binom(nn, p = 1/4)
}	
