# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 30th November 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis


### Estimate biases and variances of hat{lambda}
### for an exponential (mean 1/lambda) sample of size ssize
### when lambda_0 is true.lambda
### via simulation of size nsimu
moments.mle <- function(true.lambda = 1, ssize = 5, nsimu = 10000, tol = 1e-7) {
  objective <- function(lambda, sampl) {
    (1/lambda - mean(sampl))^2
  }
  ests.ml <- ests.bc <- objs <- numeric(nsimu)
  for (i in 1:nsimu) {
    temp <- optimize(f = objective, interval = c(0,100),
                           sampl = rexp(ssize, rate = true.lambda), tol = tol)
    objs[i] <- temp$objective
    ests.ml[i] <-  temp$minimum
    ests.bc[i] <- ests.ml[i] - ests.ml[i]/ssize
  }
  if (any(objs > tol*1000)) {
    browser()
    warning("Some fits seem not to have converged")
  }
  res <- matrix(c(mean(ests.ml - true.lambda), mean((ests.ml - true.lambda)^2),
                  mean(ests.bc - true.lambda), mean((ests.bc - true.lambda)^2)), 2, 2)
  dimnames(res) <- list(c("Bias", "MSE"), c("MLE", "BC"))
  res
}

### Estimate biases and variances of hat{lambda} for ssize = 5
### for a sequence lambdas of true values for lambda
set.seed(1)
ssize <- 5
nsimu <- 10000
lambdas <- seq(0.1, 6, 0.2)
res <- as.list(rep(0, length(lambdas)))
for (i in seq(along = lambdas)) {
  cat("Simulating for lambda =", lambdas[i], "\n")
  res[[i]] <- moments.mle(true.lambda = lambdas[i], ssize = ssize,
                          nsimu = nsimu, tol = 1e-7)
}

biases.ml <- unlist(lapply(res, function(x) x[1,1]))
biases.bc <- unlist(lapply(res, function(x) x[1,2]))
mses.ml <- unlist(lapply(res, function(x) x[2,1]))
mses.bc <- unlist(lapply(res, function(x) x[2,2]))
results <- cbind(lambdas, biases.ml, biases.bc, mses.ml, mses.bc)
results

### Plot the biases of the maximum likelihood esitmator
### and the bias-corrected estimator
plot(lambdas, biases.ml, ylim = c(-0.2, 1.5),
     xlab = expression(lambda[0]), ylab = "Estimated bias")
points(lambdas, biases.bc, pch = 16)
abline(0, 0)
title(main = paste("n =", ssize),
      sub = expression(paste("based on 10000 simulated samples for each value of ", lambda[0])))
legend("topleft", pch = c(1, 16),
       legend = c("Maximum likelihood", "Bias corrected"))


